@@ -1,6 +1,6 @@
 use crate::erasure::ErasureConfig;
 use serde::{Deserialize, Serialize};
-use solana_sdk::{clock::Slot, hash::Hash};
+use solana_sdk::clock::Slot;
 use std::{collections::BTreeSet, ops::RangeBounds};
 
 #[derive(Clone, Debug, Default, Deserialize, Serialize, Eq, PartialEq)]
@@ -75,33 +75,6 @@ pub enum ErasureMetaStatus {
     StillNeed(usize),
 }
 
-#[derive(Deserialize, Serialize, Debug, PartialEq)]
-pub enum FrozenHashVersioned {
-    Current(FrozenHashStatus),
-}
-
-impl FrozenHashVersioned {
-    pub fn frozen_hash(&self) -> Hash {
-        match self {
-            FrozenHashVersioned::Current(frozen_hash_status) => frozen_hash_status.frozen_hash,
-        }
-    }
-
-    pub fn is_duplicate_confirmed(&self) -> bool {
-        match self {
-            FrozenHashVersioned::Current(frozen_hash_status) => {
-                frozen_hash_status.is_duplicate_confirmed
-            }
-        }
-    }
-}
-
-#[derive(Deserialize, Serialize, Debug, PartialEq)]
-pub struct FrozenHashStatus {
-    pub frozen_hash: Hash,
-    pub is_duplicate_confirmed: bool,
-}
-
 impl Index {
     pub(crate) fn new(slot: Slot) -> Self {
         Index {
@@ -152,6 +125,10 @@ impl ShredIndex {
             self.set_present(idx, present);
         }
     }
+
+    pub fn largest(&self) -> Option<u64> {
+        self.index.iter().rev().next().copied()
+    }
 }
 
 impl SlotMeta {
@@ -276,11 +253,6 @@ pub struct PerfSample {
     pub sample_period_secs: u16,
 }
 
-#[derive(Clone, Debug, Default, Deserialize, Serialize, PartialEq)]
-pub struct ProgramCost {
-    pub cost: u64,
-}
-
 #[cfg(test)]
 mod test {
     use super::*;
